
import 'package:cs_resources/generated/assets.dart';
import 'package:flutter/material.dart';
import 'package:plugin_platform/engine/toast/toast_engine.dart';
import 'package:riverpod_annotation/riverpod_annotation.dart';
import 'package:router/path/router_path.dart';
import 'package:shared/utils/color_utils.dart';
import 'package:shared/utils/log_utils.dart';
import 'package:auto_route/auto_route.dart';
import 'package:shared/utils/util.dart';
import 'package:widgets/picker/option_pick_util.dart';


import 'garagesale_post_page.dart';
import 'garagesale_post_state.dart';

part 'garagesale_post_vm.g.dart';

@riverpod
class GaragesalePostVm extends _$GaragesalePostVm {

  GaragesalePostPageState initState() {
    return GaragesalePostPageState(
      fieldErrorText: '',
      formData: {
        'sale': {
          'value': '',
          'controller': TextEditingController(),
          'hintText': '',
          'focusNode': FocusNode(),
          'obsecure': false,
        },
        'category': {
          'value': '',
          'controller': TextEditingController(),
          'hintText': '',
          'focusNode': FocusNode(),
          'obsecure': false,
        },
        'title': {
          'value': '',
          'controller': TextEditingController(),
          'hintText': '',
          'focusNode': FocusNode(),
          'obsecure': false,
        },
        'description': {
          'value': '',
          'controller': TextEditingController(),
          'hintText': '',
          'focusNode': FocusNode(),
          'obsecure': false,
        },
      },
      imgList: [],
    );
  }

  @override
  GaragesalePostPageState build() {
    // 初始化状态
    GaragesalePostPageState state = initState();

    // 当前渲染完成后执行一次
    WidgetsBinding.instance!.addPostFrameCallback((_) {
      // 获取焦点
      initListener(state);
      ref.onDispose(() {
        onDispose(state);
      });
    });

    return state;
  }

  //catogery 选择选项
  void pickCategoryCallback() {
    _dismissKeyboard(keyStr: 'title');
    _dismissKeyboard(keyStr: 'description');

    OptionPickerUtil.showCupertinoOptionPicker(
      items: state.categoryOptionsList,
      initialSelectIndex: 0,
      onPickerChanged: (_, index) {
        state = state.copyWith(categorySelectedOption: state.categoryOptionsList[index]);
      },
    );
  }

  //sale 选择选项
  void pickSaleCallback() {
    _dismissKeyboard(keyStr: 'title');
    _dismissKeyboard(keyStr: 'description');

    OptionPickerUtil.showCupertinoOptionPicker(
      items: state.saleOptionsList,
      initialSelectIndex: 0,
      onPickerChanged: (_, index) {
        state = state.copyWith(saleSelectedOption: state.saleOptionsList[index]);
      },
    );
  }

  // 获取聚焦的node
  FocusNode getFocusNode(Map<String, dynamic> formData, String keyStr) {
    return formData![keyStr]!['focusNode'];
  }

  // 取消表单聚焦状态
  void _dismissKeyboard({String? keyStr}) {
    if(keyStr!.isNotEmpty){
      Log.d("FeedbackCreateViewModel 取消单个表单 $keyStr 聚焦状态");
      final FocusNode sigleItemFocusNode = state.formData[keyStr]!['focusNode'];
      sigleItemFocusNode!.unfocus();
    }else {
      Log.d("FeedbackCreateViewModel 取消所有表单聚焦状态");
      // 遍历 formData 的所有表单 然后逐一取消聚焦
      Map<String, dynamic> formData = state.formData;
      for(String key in formData.keys){
        final FocusNode sigleItemFocusNode = formData[key]['focusNode'];
        sigleItemFocusNode!.unfocus();
      }
    }
  }

  // 获取表单的值
  dynamic _getFormFieldValue({List<String>? keys, String? keyStr}) {
    if(keyStr!.isEmpty){
      if(keys!.isNotEmpty){
        // 遍历keys获取指定多个 keys 的表单value
        Map<String, dynamic> resultValueMap = {};
        for (String itemStr in keys!) {
          TextEditingController sigleItemController = state.formData[itemStr]!['controller'];
          resultValueMap[itemStr] = sigleItemController.text;
        }
        return resultValueMap;
      }
    }else if(keyStr!.isNotEmpty){
      // 获取单个表单的value
      final TextEditingController sigleItemController = state.formData[keyStr]!['controller'];
      return sigleItemController.text;
    }
  }

  ///提交反馈
  void submitGaragesalePost() {
    Log.d("GaragesalePostPageState 提交表单");
    state = state.copyWith(fieldErrorText: null);

    _dismissKeyboard();

    // 获取表单的值
    String  saleValue= _getFormFieldValue(keyStr: 'sale');
    String  categoryValue= _getFormFieldValue(keyStr: 'category');
    String  titleValue= _getFormFieldValue(keyStr: 'title');
    String  descriptionValue= _getFormFieldValue(keyStr: 'description');

    Log.d('当前待提交的 sale:$saleValue category:$categoryValue title:$titleValue description:$descriptionValue  imgList:${state.imgList}');


    if (Utils.isEmpty(saleValue)) {
      state = state.copyWith(fieldErrorText: "Title cannot be empty!");
      return;
    }

    if (Utils.isEmpty(categoryValue)) {
      state = state.copyWith(fieldErrorText: "category cannot be empty!");
      return;
    }

    //去成功页面
    // FeedbackCreateSuccessPage.startInstance();
    // 返回上一页
  }

  //选中图片
  void setImgList(List<String> list) {
    state = state.copyWith(imgList: list);
  }

  //初始化监听
  void initListener(GaragesalePostPageState initState) {

    // 获取表单的焦点节点
    final FocusNode focusNode = getFocusNode(state.formData, 'mind');

    focusNode.addListener(() {
      // 获取焦点的时候清空错误文本
      if (focusNode.hasFocus) {
        state = state.copyWith(fieldErrorText: null);
      }
    });
  }

  //销毁资源
  void onDispose(GaragesalePostPageState initState) {
    // 获取表单的焦点节点
    final FocusNode focusNode = getFocusNode(state.formData, 'mind');
    focusNode.dispose();

    Log.d("GaragesalePostPageState 销毁 onDispose");
  }


}
